Enum PrimitiveTopology

#[repr(i32)]
pub enum PrimitiveTopology { PointList = 0, LineList = 1, LineStrip = 2, TriangleList = 3, TriangleStrip = 4, TriangleFan = 5, LineListWithAdjacency = 6, LineStripWithAdjacency = 7, TriangleListWithAdjacency = 8, TriangleStripWithAdjacency = 9, PatchList = 10, }
Supported primitive topologies

Variants



PointList = 0



LineList = 1



LineStrip = 2

A series of connected line primitives with consecutive lines sharing a vertex



TriangleList = 3



TriangleStrip = 4

A series of connected triangle primitives with consecutive triangles sharing an edge



TriangleFan = 5

A series of connected triangle primitives with all triangles sharing a common vertex



LineListWithAdjacency = 6



LineStripWithAdjacency = 7

A series of connected line primitives with adjacency with consecutive primitives sharing three vertices



TriangleListWithAdjacency = 8



TriangleStripWithAdjacency = 9

Connected triangle primitives with adjacency with consecutive triangles sharing an edge



PatchList = 10