Struct InstanceCreateFlags

pub struct InstanceCreateFlags(/* private fields */);

Implementations

Source

impl InstanceCreateFlags

pub const fn bits(&self) -> VkInstanceCreateFlags

Returns bits in this flags

pub const fn has_any(self, other: Self) -> bool

Returns true if any of specified bits are contained in this flag.

pub const fn has_all(self, other: Self) -> bool

Returns true if all of specified bits are contained in this flag.

pub const fn merge(self, other: Self) -> Self

merge two flags (const alias of BitOr)

Source

impl InstanceCreateFlags

pub const EMPTY: Self

Empty bits.