Trait VulkanSinkStructure

pub unsafe trait VulkanSinkStructure {
    // Required methods
    fn as_generic(&self) -> &GenericVulkanSinkStructure;
    fn as_generic_mut(&mut self) -> &mut GenericVulkanSinkStructure;
}

Required Methods

fn as_generic(&self) -> &GenericVulkanSinkStructure

Cast this structure ref to generic one. This is same as transmute but must be safe.

fn as_generic_mut(&mut self) -> &mut GenericVulkanSinkStructure

Cast this structure mutable ref to generic one. This is same as transmute but must be safe.

Implementations on Foreign Types

Source

impl<T> VulkanSinkStructure for &mut T

Source

impl<T> VulkanSinkStructure for Box<T>

Implementors