#[repr(i32)]pub enum PrimitiveTopology {
PointList = 0,
LineList = 1,
LineStrip = 2,
TriangleList = 3,
TriangleStrip = 4,
TriangleFan = 5,
LineListWithAdjacency = 6,
LineStripWithAdjacency = 7,
TriangleListWithAdjacency = 8,
TriangleStripWithAdjacency = 9,
PatchList = 10,
}Expand description
Supported primitive topologies
Variants
PointList = 0
A series of separate point primitives
LineList = 1
A series of separate line primitives
LineStrip = 2
A series of connected line primitives with consecutive lines sharing a vertex
TriangleList = 3
A series of separate triangle primitives
TriangleStrip = 4
A series of connected triangle primitives with consecutive triangles sharing an edge
TriangleFan = 5
A series of connected triangle primitives with all triangles sharing a common vertex
LineListWithAdjacency = 6
A series of separate line primitives with adjacency
LineStripWithAdjacency = 7
A series of connected line primitives with adjacency with consecutive primitives sharing three vertices
TriangleListWithAdjacency = 8
A series of separate triangle primitives with adjacency
TriangleStripWithAdjacency = 9
Connected triangle primitives with adjacency with consecutive triangles sharing an edge
PatchList = 10